//+------------------------------------------------------------------+
//|                                                  SecondChart.mq4 |
//|                                   Copyright  2008, Serega Lykov |
//|                                       http://mtexperts.narod.ru/ |
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//|                                              SecondChart sw v1.2 |
//|                       Fausto Del Gaudio - thefxpros@katamail.com |
//|                                               Versions changelog |
//|                                        original indicator - v1.0 |
//|                                minor graphic improvements - v1.1 |
//|               added non standard timeframes compatibility - v1.2 |
//+------------------------------------------------------------------+

#property copyright "Copyright  2016, Fausto Del Gaudio"
#property link      "mailto:thefxpros@katamail.com"
#property description "Second chart sw v1.2 draws a copy of a desired chart in the mt4 subwindow."
#property description "--- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- ---"
#property description "Use 'Period' list for standard timeframe charts and 'Custom period' field for non standard "
#property description "charts (renko, range bars, tick charts, etc). In order to use multiple istances of second "
#property description "chart, set a progressive number for each in 'SubWindow ID' field."


#property strict

//---- property of indicator ----------------------------------------+
#property indicator_separate_window

#property indicator_buffers 8
#property indicator_color1 Green              // color of body, if Close > Open
#property indicator_color2 Red                // color of body, if Close < Open
#property indicator_color3 White              // color of body, if Close = Open
#property indicator_color4 Black              // color of background
#property indicator_color5 Green              // color of shadow, if Close > Open
#property indicator_color6 Red                // color of shadow, if Close < Open
#property indicator_color7 White              // color of shadow, if Close = Open
#property indicator_color8 Black              // color of background
#property indicator_width1 2                  // width of body, if Close > Open
#property indicator_width2 2                  // width of body, if Close < Open
#property indicator_width3 2                  // width of body, if Close = Open
#property indicator_width4 2                  // width of background
#property indicator_width5 1                  // width of shadow, if Close > Open
#property indicator_width6 1                  // width of shadow, if Close < Open
#property indicator_width7 1                  // width of shadow, if Close = Open
#property indicator_width8 1                  // width of background

enum WIDTH 
  {
   Size_very_tiny = 1,  // 1 - very tiny 
   Size_small     = 2,  // 2 - small
   Size_medium    = 3,  // 3 - medium
   Size_thick     = 4,  // 4 - thick
   Size_fat       = 5,  // 5 - fat
  };
  
enum TFs
  {
 // TF_CUSTOM  = -1,// Use custom period
  TF_CURRENT = 0, // Current Timeframe
  TF_M1 = 1,      // M1
  TF_M5 = 5,      // M5
  TF_M15= 15,     // M15
  TF_M30= 30,     // M30
  TF_H1 = 60,     // H1
  TF_H4 = 240,    // H4
  TF_D1 = 1440,   // Daily
  TF_W1 = 10080,  // Weekly   
  TF_MN1= 43200,  // Monthly
 };
 
//---- external parameters ------------------------------------------+
extern string PARAMETERS               = "==== SECOND CHART SW PARAMETERS ====";
extern string PairSymbol               = "EURUSD";       // Symbol for second chart
extern TFs    TimeFrame                = PERIOD_CURRENT; // Period for second chart
extern int    CustomTF                 = 0;             // Custom period for second chart
extern bool   ShowBidLine              = true;           // Show bid line
extern bool   ShowBidLabel             = true;           // Show bid price label
extern string SecondChart_ID           = "1";            // SubWindow ID (set different numbers for each istance)
extern int    OffsetLabel              = 15;             // Offset (from current bar) for bid price label
extern int    CountBars                = 200;            // N of bars to calculate (0 = all bars)

extern string GRAPHICS_COLORS          = "==== SECOND CHART SW COLORS ====";
extern color  BidLine_Color            = LightSlateGray; // Color of bid line
extern color  Body_BullCandle_Color    = LimeGreen;      // Color of BULL candles BODY
extern color  Body_BearCandle_Color    = Red;            // Color of BEAR candles BODY
extern color  Body_DojiCandle_Color    = Black;          // Color of DOJI candles BODY
extern color  Wick_BullCandle_Color    = SeaGreen;       // Color of BULL candles WICK
extern color  Wick_BearCandle_Color    = Crimson;        // Color of BEAR candles WICK
extern color  Wick_DojiCandle_Color    = Black;          // Color of DOJI candles WICK

extern string GRAPHICS_SIZES           = "==== SECOND CHART SW OBJECTS SIZES ====";
extern int    BidLabel_Font_Size       = 8;              // Font size for bid price label
extern ENUM_LINE_STYLE BidLine_Style   = 0;              // Style of bid line 
extern WIDTH  Body_Candles_Size        = 2;              // Width of candles body
extern WIDTH  Wick_Candles_Size        = 1;              // Width of candles wick

//---- buffers ------------------------------------------------------+
static double UpBodyBuffer[];
static double DnBodyBuffer[];
static double EqBodyBuffer[];
static double BgBodyBuffer[];
static double UpShadowBuffer[];
static double DnShadowBuffer[];
static double EqShadowBuffer[];
static double BgShadowBuffer[];

//---- global variables ---------------------------------------------+
string period;
int    limit;
double open,close,high,low;
bool   recalculate;
int    digits;
int    offset_label;
int    attempts;
double point;
string symbol;
string short_name;
string Bid_line = StringConcatenate("Bid_line",SecondChart_ID);
string Bid_label = StringConcatenate("Bid_label",SecondChart_ID);

//---- initialization of indicator ----------------------------------+
int init()
  {
     
          
  switch(TimeFrame)
   {
      case 1 : period="Period M1"; break;
      case 5 : period="Period M5"; break;
      case 15 : period="Period M15"; break;
      case 30 : period="Period M30"; break;
      case 60 : period="Period H1"; break;
      case 240 : period="Period H4"; break;
      case 1440 : period="Period D1"; break;
      case 10080 : period="Period W1"; break;
      case 43200 : period="Period MN1"; break;
      default : period= "Period "+IntegerToString(Period(),0,0)+"";
   }
   
   if (CustomTF != 0) 
   {
      TimeFrame = CustomTF;
      period = "Custom timeframe " + IntegerToString(CustomTF,0,0);
   }     
       
   //---- set a "short" name of the indicator -----------------------+
   short_name = ("SC sw 1.2 -- ["+PairSymbol+" - "+period+" - SubWindow ID "+SecondChart_ID+"] ");
   IndicatorShortName(short_name);
   
   
              
    //---- set a constants -------------------------------------------+ 
    
   color bg=(color)ChartGetInteger(ChartID(),CHART_COLOR_BACKGROUND);
   int len_symbol  = StringLen(Symbol());
   int len_curpair = StringLen(PairSymbol);
   if(len_curpair >= len_symbol) symbol = StringSubstr(PairSymbol,0,len_symbol);
   else symbol = StringConcatenate(PairSymbol,StringSubstr(Symbol(),len_curpair,len_symbol));
   point = MarketInfo(symbol,MODE_POINT);
   digits = MarketInfo(symbol,MODE_DIGITS);
   offset_label = Period() * 60 * OffsetLabel;
   recalculate = true;
   attempts = 10;
   
   
   
   
   //---- set a accuracy of values of the indicator -----------------+   
   IndicatorDigits(digits);
   
   //---- set a style for line --------------------------------------+
   SetIndexStyle(0,DRAW_HISTOGRAM,STYLE_SOLID,Body_Candles_Size,Body_BullCandle_Color);
   SetIndexStyle(1,DRAW_HISTOGRAM,STYLE_SOLID,Body_Candles_Size,Body_BearCandle_Color);
   SetIndexStyle(2,DRAW_HISTOGRAM,STYLE_SOLID,Body_Candles_Size,Body_DojiCandle_Color);
   SetIndexStyle(3,DRAW_HISTOGRAM,STYLE_SOLID,Body_Candles_Size,bg);
   SetIndexStyle(4,DRAW_HISTOGRAM,STYLE_SOLID,Wick_Candles_Size,Wick_BullCandle_Color);
   SetIndexStyle(5,DRAW_HISTOGRAM,STYLE_SOLID,Wick_Candles_Size,Wick_BearCandle_Color);
   SetIndexStyle(6,DRAW_HISTOGRAM,STYLE_SOLID,Wick_Candles_Size,Wick_DojiCandle_Color);
   SetIndexStyle(7,DRAW_HISTOGRAM,STYLE_SOLID,Wick_Candles_Size,bg);
   
   //---- set a arrays for line -------------------------------------+
   SetIndexBuffer(0,UpBodyBuffer);
   SetIndexBuffer(1,DnBodyBuffer);
   SetIndexBuffer(2,EqBodyBuffer);
   SetIndexBuffer(3,BgBodyBuffer);
   SetIndexBuffer(4,UpShadowBuffer);
   SetIndexBuffer(5,DnShadowBuffer);
   SetIndexBuffer(6,EqShadowBuffer);
   SetIndexBuffer(7,BgShadowBuffer);
   
   
   //---- finish of initialization ----------------------------------+
   return(0);
  }

//---- deinitialization of indicator --------------------------------+
int deinit()
  {
   if(ShowBidLine)  ObjectDelete(Bid_line);
   if(ShowBidLabel) ObjectDelete(Bid_label);
   return(0);
  }

//---- SecondChart --------------------------------------------------+
int start()
  {
   //---- amount not changed bars after last call of the indicator --+
   int counted_bars = IndicatorCounted();
   if(counted_bars < 0) return(-1);
   if(recalculate)
     {
      limit = Bars;
      recalculate = false;
     }
   else limit = Bars - counted_bars;
   if(CountBars > 0) if(limit > CountBars) limit = CountBars;
   
   //---- set values of candles -------------------------------------+
   for(int i=limit; i>=0; i--)
     {
      open  = NormalizeDouble(iOpen(symbol,TimeFrame,i),digits);
      close = NormalizeDouble(iClose(symbol,TimeFrame,i),digits);
      high  = NormalizeDouble(iHigh(symbol,TimeFrame,i),digits);
      low   = NormalizeDouble(iLow(symbol,TimeFrame,i),digits);
      if(open == 0 || close == 0 || high == 0 || low == 0)
        {
         if(attempts > 0)
           {
            attempts--;
            recalculate = true;
            return(0);
           }
        }
      if(open < close)
        {
         UpBodyBuffer[i] = close;
         DnBodyBuffer[i] = EMPTY_VALUE;
         EqBodyBuffer[i] = EMPTY_VALUE;
         BgBodyBuffer[i] = open;
         UpShadowBuffer[i] = high;
         DnShadowBuffer[i] = EMPTY_VALUE;
         EqShadowBuffer[i] = EMPTY_VALUE;
         BgShadowBuffer[i] = low;
        }
      else
        {
         if(open > close)
           {
            UpBodyBuffer[i] = EMPTY_VALUE;
            DnBodyBuffer[i] = open;
            EqBodyBuffer[i] = EMPTY_VALUE;
            BgBodyBuffer[i] = close;
            UpShadowBuffer[i] = EMPTY_VALUE;
            DnShadowBuffer[i] = high;
            EqShadowBuffer[i] = EMPTY_VALUE;
            BgShadowBuffer[i] = low;
           }
         else
           {
            UpBodyBuffer[i] = EMPTY_VALUE;
            DnBodyBuffer[i] = EMPTY_VALUE;
            EqBodyBuffer[i] = open;
            BgBodyBuffer[i] = open - point;
            UpShadowBuffer[i] = EMPTY_VALUE;
            DnShadowBuffer[i] = EMPTY_VALUE;
            if(high == low)
              {
               EqShadowBuffer[i] = EMPTY_VALUE;
               BgShadowBuffer[i] = EMPTY_VALUE;
              }
            else
              {
               EqShadowBuffer[i] = high;
               BgShadowBuffer[i] = low;
              }
           }
        }
     }
     
   //---- draw Bid line ---------------------------------------------+   
      if(ShowBidLine)
     {
   //   if(ObjectFind(Bid_line) < 0)
      //  {
         ObjectCreate(Bid_line,OBJ_HLINE,WindowFind(short_name),0,close);
         ObjectSet(Bid_line,OBJPROP_STYLE,BidLine_Style);
         ObjectSet(Bid_line,OBJPROP_COLOR,BidLine_Color);
      //  }
      double obj_level = NormalizeDouble(ObjectGet(Bid_line,OBJPROP_PRICE1),digits);
      if(obj_level != close) ObjectMove(Bid_line,0,Time[0],close);
     }
     
   //---- show Bid value --------------------------------------------+
   if(ShowBidLabel)
     {
      datetime dt_offset = Time[0] + offset_label;
      if(ObjectFind(Bid_label) < 0)
        {
         ObjectCreate(Bid_label,OBJ_TEXT,WindowFind(short_name),dt_offset,close);
         ObjectSetText(Bid_label,DoubleToStr(close,digits),BidLabel_Font_Size,"Arial",BidLine_Color);
        }
      int obj_level = NormalizeDouble(ObjectGet(Bid_label,OBJPROP_PRICE1),digits);
      if(ObjectGet(Bid_label,OBJPROP_TIME1) != dt_offset || obj_level != close)
        {
         ObjectMove(Bid_label,0,dt_offset,close);
         ObjectSetText(Bid_label,DoubleToStr(close,digits),BidLabel_Font_Size,"Arial",BidLine_Color);
        }
     }
     
   //---- finish of iteration ---------------------------------------+
   return(0);
  }

